# frozen_string_literal: true

module Safrano
  module Filter
    class Parser
      # Input tokenizer
      module Token
        FUNCNAMES = %w[concat substringof endswith startswith length indexof
                       replace substring trim toupper tolower
                       day hour minute month second year
                       round floor ceiling].freeze
        FUNCRGX = FUNCNAMES.join('|')
        NULLRGX = 'null|NULL|Null'
        QSTRINGRGX = /'((?:[^']|(?:'{2}))*)'/.freeze
        BINOBOOL = '[eE][qQ]|[LlgGNn][eETt]|[aA][nN][dD]|[oO][rR]'
        BINOARITHM = '[aA][dD][dD]|[sS][uU][bB]|[mM][uU][lL]|[dD][iI][vV]|[mM][oO][dD]'
        NOTRGX = 'not|NOT|Not'
        FPRGX = '\d+(?:\.\d+)?(?:e[+-]?\d+)?'
        QUALITRGX = '\w+(?:\/\w+)+'
        RGX = /(#{FUNCRGX})|(#{NULLRGX})|([\(\),])|(#{BINOBOOL})\s+|(#{BINOARITHM})|(#{NOTRGX})|#{QSTRINGRGX}|(#{FPRGX})|(#{QUALITRGX})|(\w+)|(')/.freeze

        def each_typed_token(inp)
          typ = nil

          inp.scan(RGX) do |groups|
            idx = nil
            found = nil
            groups.each_with_index do |tok, i|
              if (found = tok)
                idx = i
                break
              end
            end
            typ = case idx
                  when 0
                    :FuncTree
                  when 1
                    :NullLiteral
                  when 2
                    case found
                    when '(', ')'
                      :Delimiter
                    when ','
                      :Separator
                    end
                  when 3
                    :BinopBool
                  when 4
                    :BinopArithm
                  when 5
                    :UnopTree
                  when 6
                    :QString
                  when 7
                    :FPNumber
                  when 8
                    :Qualit
                  when 9
                    :Literal
                  when 10
                    :unmatchedQuote
                  end
            yield found, typ
          end
        end
      end
    end
  end
end
